﻿using System;
using System.Threading.Tasks;
using System.Windows.Controls;
using Buddy.Coroutines;
using log4net;
using Loki.Bot;
using Loki.Bot.Logic.Bots.OldGrindBot;
using Loki.Common;
using Loki.Game;
using Loki.Game.GameData;
using Loki.Game.Objects.Items;
using Utility = Loki.Bot.Logic.Bots.OldGrindBot.Utility;
using System.IO;
using System.Windows.Markup;
using System.Windows.Data;
using Loki.Bot.Pathfinding;
using System.Diagnostics;


namespace DockCorruptedFarm
{
    internal class DockCorruptedFarm : IPlugin
    {
        public static readonly ILog Log = Logger.GetLoggerInstanceForType();
        public static TaskManager GrindBotTaskManager = GetGrindBotTaskManager();
        private TaskManager _taskManager;

        public static event Action<uint, uint, DatWorldAreaWrapper> AreaChanged;
        public static event Action PlayerDied;

        private static readonly CustomLogger CustomLogger = new CustomLogger("Logs\\DockCorruptedFarm", "DockCorruptedFarm", null, null);
        private bool _enabled = true;
        // <summary>User's can change this to make the plugin mover closer or further away from a cached location.</summary>
        private const int Range = 15;

        public async Task<bool> Logic(string type, params dynamic[] param)
        {
            
            if (type == "core_player_died_event")
            {
                EventInvocators.RaisePlayerDiedEvent();
            }
            return false;
        }

        public static class EventInvocators
        {

            public static void RaiseAreaChangedEvent(uint oldSeed, uint newSeed, DatWorldAreaWrapper newArea)
            {
                var handler = AreaChanged;
                if (handler != null) handler(oldSeed, newSeed, newArea);
            }

            public static void RaisePlayerDiedEvent()
            {
                var handler = PlayerDied;
                if (handler != null) handler();
            }
        }

        public object Execute(string name, params dynamic[] param)
        {
            return null;
        }
        // <summary> The name of the plugin. </summary>
        public static TaskManager GetGrindBotTaskManager()
        {
            var bot = BotManager.CurrentBot;
            return (TaskManager)bot.Execute("GetTaskManager");
        }
        public static void AddTask(ITask task, string taskToProcess = "", bool front = true, bool after = false, bool before = false)
        {
            if (front)
            {
                if (!GrindBotTaskManager.AddAtFront(task))
                    Log.ErrorFormat("[{0}] AddAtFront failed for {1}", "AddTask", task.Name);
                else
                    Log.DebugFormat("[{0}] {1} Sucessfully added at front of the list", "AddTask", task.Name);
            }

            if (after)
            {
                if (!GrindBotTaskManager.AddAfter(task, taskToProcess))
                    Log.ErrorFormat("[{0}] AddAfter failed for {1}", "AddTask", task.Name);
                else
                    Log.DebugFormat("[{0}] {1} Sucessfully added after {2}", "AddTask", task.Name, taskToProcess);
            }

            if (before)
            {
                if (!GrindBotTaskManager.AddBefore(task, taskToProcess))
                    Log.ErrorFormat("[{0}] AddBefore failed for {1}", "AddTask", task.Name);
                else
                    Log.DebugFormat("[{0}] {1} Sucessfully added before {2}", "AddTask", task.Name, taskToProcess);
            }
        }



        // <summary> The plugin start callback. Do any initialization here. </summary>
        public void Start()
        {
            // if (!LokiPoe.IsInGame && !LokiPoe.CurrentWorldArea.IsMap)
            // {

            //     EventInvocators.RaiseMapEnteredEvent();

            // }
            _taskManager = (TaskManager)BotManager.CurrentBot.Execute("GetTaskManager");
            if (_taskManager == null)
            {
                Log.Error("[MapRunner] Fail to get the global TaskManager.");
                BotManager.Stop();
                return;
            }

            AddTask(new TrackCorrupted(), "ExploreTask", front: false, after: false, before: true);
            Log.DebugFormat("[DockCorruptedFarm] added before exploretask");


            _taskManager.Remove("ReturnToGrindZoneTask");
            _taskManager.Remove("UseLooseCandleTask");
            _taskManager.Remove("HandleA2Q9");
            _taskManager.Remove("HandleA2Q9b");
            _taskManager.Remove("HandleA2Q11");
            _taskManager.Remove("HandleA3Q1");
            _taskManager.Remove("HandleSewers");
            _taskManager.Remove("HandleUndyingBlockage");
            _taskManager.Remove("HandleLockedDoor");
            _taskManager.Remove("HandleDeshretSeal");
            _taskManager.Remove("HandleGlyphWall");
            _taskManager.Remove("TravelToGrindZoneTask");
            _taskManager.Remove("TravelThroughBossAreasTask-BossFarm");
            _taskManager.Remove("EnterArenaTask");
            _taskManager.Remove("TravelThroughBossAreasTask-PassThrough");
            _taskManager.Remove("ExplorationCompleteTask (Early)");
            _taskManager.Remove("TrackMoreMobs");
            _taskManager.Remove("HandleMerveilArea");
            _taskManager.Remove("HandlePietyArea");
            _taskManager.Remove("HandleDominusArea");
            _taskManager.Remove("HandleKaomArea");
            _taskManager.Remove("HandleDaressoArea");
            _taskManager.Remove("HandleWeaverArea");


        }

        public string Name
        {
            get { return "DockCorruptedFarm"; }
        }


        // <summary> The description of the plugin. </summary>
        public string Description
        {
            get { return "A plugin that skips dock that don't have corrupted instance."; }
        }

        /// <summary>The author of the plugin.</summary>
        public string Author
        {
            get { return "JYam"; }
        }

        // <summary>The version of the plugin.</summary>
        public string Version
        {
            get { return "(Unfinished and has code smells)"; }
        }

        // <summary>Initializes this plugin.</summary>
        public void Initialize()
        {
            Log.DebugFormat("[DockCorruptedFarm] Initialize");
        }

        // <summary> The plugin tick callback. Do any update logic here. </summary>
        public void Tick()
        {
        }

        // <summary> The plugin stop callback. Do any pre-dispose cleanup here. </summary>
        public void Stop()
        {
            Log.DebugFormat("[DockCorruptedFarm] Stop");
        }


        #region Implementation of IEnableable

        // <summary>Is this plugin currently enabled?</summary>
        public bool IsEnabled
        {
            get { return _enabled; }
        }
        public void Deinitialize()
        {
            Log.Debug("[DockCorruptedFarm] Deinitialize");
        }

        // <summary> The plugin is being enabled.</summary>
        public void Enable()
        {
            Log.DebugFormat("[DockCorruptedFarm] Enable");
            _enabled = true;
        }

        // <summary> The plugin is being disabled.</summary>
        public void Disable()
        {
            Log.DebugFormat("[DockCorruptedFarm] Disable");
            _enabled = false;
        }
        public JsonSettings Settings
        {
            get { return null; }
        }

        public UserControl Control
        {
            get
            {
                using (var fs = new FileStream(@"Plugins\DockCorruptedFarm\SettingsGui.xaml", FileMode.Open))
                {
                    var root = (UserControl)XamlReader.Load(fs);
                    var instance = DockCorruptedFarmSettings.Instance;

                    if (!Wpf.SetupLabelBinding(root, "fragDuskLabel", "fragDusk", BindingMode.OneWay, instance))
                    {
                        Log.DebugFormat(
                            "[SettingsControl] SetupTextBoxBinding failed for 'fragDuskLabel'.");
                        throw new Exception("The SettingsControl could not be created.");
                    }
                    if (!Wpf.SetupLabelBinding(root, "fragDawnLabel", "fragDawn", BindingMode.OneWay, instance))
                    {
                        Log.DebugFormat(
                            "[SettingsControl] SetupTextBoxBinding failed for 'fragDawnLabel'.");
                        throw new Exception("The SettingsControl could not be created.");
                    }
                    if (!Wpf.SetupLabelBinding(root, "fragNoonLabel", "fragNoon", BindingMode.OneWay, instance))
                    {
                        Log.DebugFormat(
                            "[SettingsControl] SetupTextBoxBinding failed for 'fragNoonLabel'.");
                        throw new Exception("The SettingsControl could not be created.");
                    }
                    if (!Wpf.SetupLabelBinding(root, "fragMidnightLabel", "fragMidnight", BindingMode.OneWay, instance))
                    {
                        Log.DebugFormat(
                            "[SettingsControl] SetupTextBoxBinding failed for 'fragMidnightLabel'.");
                        throw new Exception("The SettingsControl could not be created.");
                    }
                    if (!Wpf.SetupLabelBinding(root, "vaalGemsLabel", "vaalGems", BindingMode.OneWay, instance))
                    {
                        Log.DebugFormat(
                            "[SettingsControl] SetupTextBoxBinding failed for 'vaalGemsLabel'.");
                        throw new Exception("The SettingsControl could not be created.");
                    }

                    return root;
                }
            }
        }


        #endregion

        #region Implementation of IDisposable

        /// <summary> </summary>
        public void Dispose()
        {
        }

        #endregion

        #region Override of Object

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return Name + ": " + Description;
        }

        #endregion

        public class TrackCorrupted : ITask
        {
            private readonly Stopwatch _sw = new Stopwatch();

            private bool _enabled = true;
            private bool _hasCorrupted = false;
            public bool count = true;
            public DockCorruptedFarmSettings instance = DockCorruptedFarmSettings.Instance;



            /// <summary>The name of this task.</summary>
            public string Name
            {
                get { return "TrackCorrupted"; }
            }

            /// <summary>A description of what this task does.</summary>
            public string Description
            {
                get { return "This task helps BasicGrindBot seek out corrupted area in The Docks."; }
            }

            /// <summary>The author of this task.</summary>
            public string Author
            {
                get { return "JYam"; }
            }

            /// <summary>The version of this task.</summary>
            public string Version
            {
                get { return "(Unfinished and has code smells)"; }
            }

            private Vector2i FindWalkablePathToTerrainObject(int i, int j)
            {
                Vector2i pos = new Vector2i(i * 23 + 10, j * 23 + 10);
                return ExilePather.WalkablePositionFor(pos, 10);

            }


            /// <summary>
            /// The tasks's execution logic.
            /// </summary>
            /// <returns>true if the TaskManager should not execute any other tasks, and false if it should.</returns>
            public async Task<bool> Logic(string type, params dynamic[] param)
            {
                if (type == "core_area_changed_event")
                {
                    uint oldSeed = (uint)param[0];
                    uint newSeed = (uint)param[1];
                    var newArea = (DatWorldAreaWrapper)param[3];
                    EventInvocators.RaiseAreaChangedEvent(oldSeed, newSeed, newArea);
                    _hasCorrupted = false;
                }
                
                if (LokiPoe.Me.IsInTown || LokiPoe.Me.IsInHideout)
                    await Coroutines.TakeWaypointTo("3_3_9", true, -1);



                if (LokiPoe.CurrentWorldArea.Name != "The Docks")
                    return false;

                LokiPoe.TerrainDataEntry[,] tgtEntries = LokiPoe.TerrainData.GetTgtEntries();
                for (int i = 0; i < tgtEntries.GetLength(0); i++)
                {
                    for (int j = 0; j < tgtEntries.GetLength(1); j++)
                    {
                        LokiPoe.TerrainDataEntry terrainDataEntry = tgtEntries[i, j];
                        if (terrainDataEntry != null)
                        //Log.InfoFormat(terrainDataEntry.TgtName);
                        {
                            if (terrainDataEntry.TgtName == "Art/Models/Terrain/Act3/AreaTransitions/act3_docks_v01_01_c1r1.tgt")
                            {
                                _hasCorrupted = true;
                                Log.InfoFormat("[TrackCorrupted] area corrupted, start to find path");
                                if (LokiPoe.Me.Position.Distance(FindWalkablePathToTerrainObject(i, j)) > Range)
                                {
                                    if (!await Coroutines.MoveToLocation(FindWalkablePathToTerrainObject(i, j), 10, 60000))
                                    {
                                        Log.ErrorFormat("[TrackCorrupted] MoveTowards failed for {0}.", FindWalkablePathToTerrainObject(i, j));
                                    }
                                    else
                                    {
                                        Log.InfoFormat("[TrackCorrupted] We have arrived.");

                                        await Coroutines.TakeAreaTransition(LokiPoe.ObjectManager.CorruptedAreaTransition(), false, 5, true);
                                        
                                    }

                                }
                                else
                                {
                                    Log.InfoFormat("[TrackCorrupted] We have arrived.");
                                    await Coroutines.TakeAreaTransition(LokiPoe.ObjectManager.CorruptedAreaTransition(), false, 5, true);
                                   
                                }
                            }
                        }
                    }
                }
                if (!_hasCorrupted)
                {
                    Log.InfoFormat("[TrackCorrupted] This dock has no corrupted area, remaking instance.");
                    await Coroutines.TakeWaypointTo("3_3_9", true, -1);
                }
                return _hasCorrupted;
            }

            /// <summary>The bot Start event.</summary>
            public void Start()
            {


                OldGrindBot.OnLoot += BasicGrindBot_OnLoot;
            }



            void BasicGrindBot_OnLoot(object sender, OldGrindBot.OnLootEventArgs e)
            {
                if (e.Name.Contains("Sacrifice at Dusk"))
                    DockCorruptedFarmSettings.Instance.IncreaseDusk();
                if (e.Name.Contains("Sacrifice at Dawn"))
                    DockCorruptedFarmSettings.Instance.IncreaseDawn();
                if (e.Name.Contains("Sacrifice at Noon"))
                    DockCorruptedFarmSettings.Instance.IncreaseNoon();
                if (e.Name.Contains("Sacrifice at Midnight"))
                    DockCorruptedFarmSettings.Instance.IncreaseMidnight();
                if (e.Name.Contains("Vaal") && e.Rarity.ToString() == "Gem")
                    DockCorruptedFarmSettings.Instance.IncreaseGems();
            }

            /// <summary>The bot Tick event.</summary>
            public void Tick()
            {

            }


            /// <summary>The bot Stop event.</summary>
            public void Stop()
            {

                OldGrindBot.OnLoot -= BasicGrindBot_OnLoot;
            }

            /// <summary>Is this task currently enabled?</summary>
            public bool IsEnabled
            {
                get { return _enabled; }
            }

            /// <summary>Called when the task should be enabled.</summary>
            public void Enable()
            {
                _enabled = true;
            }

            /// <summary>Called when the task should be disabled.</summary>
            public void Disable()
            {
                _enabled = false;
            }

            public object Execute(string name, params dynamic[] param)
            {
                return null;
            }


        }
    }
}
